@@ -12,8 +12,10 @@
 //                                                            ///
 /////////////////////////////////////////////////////////////////
 
-die('Due to a security issue, this demo has been disabled. It can be enabled by removing line '.__LINE__.' in demos/'.basename(__FILE__));
-
+die('For security reasons, this demo has been disabled. It can be enabled by removing line '.__LINE__.' in demos/'.basename(__FILE__));
+define('GETID3_DEMO_BROWSE_ALLOW_EDIT_LINK',   false);
+define('GETID3_DEMO_BROWSE_ALLOW_DELETE_LINK', false);
+define('GETID3_DEMO_BROWSE_ALLOW_MD5_LINK',    false);
 
 /////////////////////////////////////////////////////////////////
 // die if magic_quotes_runtime or magic_quotes_gpc are set
@@ -23,6 +25,9 @@ if (function_exists('get_magic_quotes_runtime') && get_magic_quotes_runtime()) {
 if (function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc()) {
 	die('magic_quotes_gpc is enabled, getID3 will not run.');
 }
+if (!defined('ENT_SUBSTITUTE')) { // defined in PHP v5.4.0
+	define('ENT_SUBSTITUTE', ENT_QUOTES);
+}
 /////////////////////////////////////////////////////////////////
 
 $PageEncoding = 'UTF-8';
@@ -31,8 +36,8 @@ $writescriptfilename = 'demo.write.php';
 
 require_once('../getid3/getid3.php');
 
-// Needed for windows only
-define('GETID3_HELPERAPPSDIR', 'C:/helperapps/');
+// Needed for windows only. Leave commented-out to auto-detect, only define here if auto-detection does not work properly
+//define('GETID3_HELPERAPPSDIR', 'C:\\helperapps\\');
 
 // Initialize getID3 engine
 $getID3 = new getID3;
@@ -89,6 +94,7 @@ if (isset($_REQUEST['filename'])) {
 	//));
 	$ThisFileInfo = $getID3->analyze($_REQUEST['filename']);
 	$AutoGetHashes = (bool) (isset($ThisFileInfo['filesize']) && ($ThisFileInfo['filesize'] > 0) && ($ThisFileInfo['filesize'] < (50 * 1048576))); // auto-get md5_data, md5_file, sha1_data, sha1_file if filesize < 50MB, and NOT zero (which may indicate a file>2GB)
+	$AutoGetHashes = ($AutoGetHashes && GETID3_DEMO_BROWSE_ALLOW_MD5_LINK);
 	if ($AutoGetHashes) {
 		$ThisFileInfo['md5_file']  = md5_file($_REQUEST['filename']);
 		$ThisFileInfo['sha1_file'] = sha1_file($_REQUEST['filename']);
@@ -103,13 +109,13 @@ if (isset($_REQUEST['filename'])) {
 	if (GETID3_OS_ISWINDOWS) {
 		// this mostly just gives a consistant look to Windows and *nix filesystems
 		// (windows uses \ as directory seperator, *nix uses /)
-		$listdirectory = str_replace('\\', '/', $listdirectory.'/');
+		$listdirectory = str_replace(DIRECTORY_SEPARATOR, '/', $listdirectory.'/');
 	}
 
 	if (strstr($_REQUEST['filename'], 'http://') || strstr($_REQUEST['filename'], 'ftp://')) {
 		echo '<i>Cannot browse remote filesystems</i><br>';
 	} else {
-		echo 'Browse: <a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory), ENT_QUOTES).'">'.getid3_lib::iconv_fallback('ISO-8859-1', 'UTF-8', $listdirectory).'</a><br>';
+		echo 'Browse: <a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">'.getid3_lib::iconv_fallback('ISO-8859-1', 'UTF-8', $listdirectory).'</a><br>';
 	}
 
 	getid3_lib::ksort_recursive($ThisFileInfo);
@@ -126,7 +132,7 @@ if (isset($_REQUEST['filename'])) {
 	if (GETID3_OS_ISWINDOWS) {
 		// this mostly just gives a consistant look to Windows and *nix filesystems
 		// (windows uses \ as directory seperator, *nix uses /)
-		$currentfulldir = str_replace('\\', '/', $listdirectory.'/');
+		$currentfulldir = str_replace(DIRECTORY_SEPARATOR, '/', $listdirectory.'/');
 	}
 
 	ob_start();
@@ -158,7 +164,7 @@ if (isset($_REQUEST['filename'])) {
 				case '..':
 					$ParentDir = realpath($file.'/..').'/';
 					if (GETID3_OS_ISWINDOWS) {
-						$ParentDir = str_replace('\\', '/', $ParentDir);
+						$ParentDir = str_replace(DIRECTORY_SEPARATOR, '/', $ParentDir);
 					}
 					$DirectoryContents[$currentfulldir]['dir'][$file]['filename'] = $ParentDir;
 					continue 2;
@@ -179,14 +185,14 @@ if (isset($_REQUEST['filename'])) {
 
 			} elseif ($TargetObjectType == 'file') {
 
-				$getID3->setOption(array('option_md5_data' => isset($_REQUEST['ShowMD5'])));
+				$getID3->setOption(array('option_md5_data' => (isset($_REQUEST['ShowMD5']) && GETID3_DEMO_BROWSE_ALLOW_MD5_LINK)));
 				$fileinformation = $getID3->analyze($currentfilename);
 
 				getid3_lib::CopyTagsToComments($fileinformation);
 
 				$TotalScannedFilesize += (isset($fileinformation['filesize']) ? $fileinformation['filesize'] : 0);
 
-				if (isset($_REQUEST['ShowMD5'])) {
+				if (isset($_REQUEST['ShowMD5']) && GETID3_DEMO_BROWSE_ALLOW_MD5_LINK) {
 					$fileinformation['md5_file'] = md5_file($currentfilename);
 				}
 
@@ -226,18 +232,20 @@ if (isset($_REQUEST['filename'])) {
 					echo '<tr bgcolor="#'.(($rowcounter++ % 2) ? $getID3checkColor_DirectoryLight : $getID3checkColor_DirectoryDark).'">';
 					if ($filename == '..') {
 						echo '<td colspan="'.$columnsintable.'">';
-						echo '<form action="'.htmlentities($_SERVER['PHP_SELF'], ENT_QUOTES).'" method="get">';
+						echo '<form action="'.htmlentities($_SERVER['PHP_SELF'], ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" method="get">';
 						echo 'Parent directory: ';
 						echo '<input type="text" name="listdirectory" size="50" style="background-color: '.$getID3checkColor_DirectoryDark.';" value="';
 						if (GETID3_OS_ISWINDOWS) {
-							echo htmlentities(str_replace('\\', '/', realpath($dirname.$filename)), ENT_QUOTES);
+							echo htmlentities(str_replace(DIRECTORY_SEPARATOR, '/', realpath($dirname.$filename)), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1');
 						} else {
-							echo htmlentities(realpath($dirname.$filename), ENT_QUOTES);
+							echo htmlentities(realpath($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1');
 						}
 						echo '"> <input type="submit" value="Go">';
 						echo '</form></td>';
 					} else {
-						echo '<td colspan="'.$columnsintable.'"><a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($dirname.$filename), ENT_QUOTES).'"><b>'.htmlentities($filename).'</b></a></td>';
+						$escaped_filename = htmlentities($filename, ENT_SUBSTITUTE, 'ISO-8859-1'); // do filesystems always return filenames in ISO-8859-1?
+						$escaped_filename = ($escaped_filename ? $escaped_filename : rawurlencode($filename));
+						echo '<td colspan="'.$columnsintable.'"><a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'"><b>'.$escaped_filename.'</b></a></td>';
 					}
 					echo '</tr>';
 				}
@@ -251,31 +259,33 @@ if (isset($_REQUEST['filename'])) {
 			echo '<th>Bitrate</th>';
 			echo '<th>Artist</th>';
 			echo '<th>Title</th>';
-			if (isset($_REQUEST['ShowMD5'])) {
-				echo '<th>MD5&nbsp;File (File) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES).'">disable</a>)</th>';
-				echo '<th>MD5&nbsp;Data (File) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES).'">disable</a>)</th>';
-				echo '<th>MD5&nbsp;Data (Source) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES).'">disable</a>)</th>';
+			if (isset($_REQUEST['ShowMD5']) && GETID3_DEMO_BROWSE_ALLOW_MD5_LINK) {
+				echo '<th>MD5&nbsp;File (File) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">disable</a>)</th>';
+				echo '<th>MD5&nbsp;Data (File) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">disable</a>)</th>';
+				echo '<th>MD5&nbsp;Data (Source) (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.'), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">disable</a>)</th>';
 			} else {
-				echo '<th colspan="3">MD5&nbsp;Data (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.').'&ShowMD5=1', ENT_QUOTES).'">enable</a>)</th>';
+				echo '<th colspan="3">MD5&nbsp;Data'.(GETID3_DEMO_BROWSE_ALLOW_MD5_LINK ?' (<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.rawurlencode(isset($_REQUEST['listdirectory']) ? $_REQUEST['listdirectory'] : '.').'&ShowMD5=1', ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">enable</a>)' : '').'</th>';
 			}
 			echo '<th>Tags</th>';
 			echo '<th>Errors &amp; Warnings</th>';
-			echo '<th>Edit</th>';
-			echo '<th>Delete</th>';
+			echo (GETID3_DEMO_BROWSE_ALLOW_EDIT_LINK   ? '<th>Edit</th>'   : '');
+			echo (GETID3_DEMO_BROWSE_ALLOW_DELETE_LINK ? '<th>Delete</th>' : '');
 			echo '</tr>';
 
 			if (isset($DirectoryContents[$dirname]['known']) && is_array($DirectoryContents[$dirname]['known'])) {
 				uksort($DirectoryContents[$dirname]['known'], 'MoreNaturalSort');
 				foreach ($DirectoryContents[$dirname]['known'] as $filename => $fileinfo) {
 					echo '<tr bgcolor="#'.(($rowcounter++ % 2) ? $getID3checkColor_FileDark : $getID3checkColor_FileLight).'">';
-					echo '<td><a href="'.htmlentities($_SERVER['PHP_SELF'].'?filename='.urlencode($dirname.$filename), ENT_QUOTES).'" title="View detailed analysis">'.htmlentities($filename).'</a></td>';
+					$escaped_filename = htmlentities($filename, ENT_SUBSTITUTE, 'ISO-8859-1');
+					$escaped_filename = ($escaped_filename ? $escaped_filename : rawurlencode($filename));
+					echo '<td><a href="'.htmlentities($_SERVER['PHP_SELF'].'?filename='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" title="View detailed analysis">'.$escaped_filename.'</a></td>';
 					echo '<td align="right">&nbsp;'.number_format($fileinfo['filesize']).'</td>';
 					echo '<td align="right">&nbsp;'.NiceDisplayFiletypeFormat($fileinfo).'</td>';
 					echo '<td align="right">&nbsp;'.(isset($fileinfo['playtime_string']) ? $fileinfo['playtime_string'] : '-').'</td>';
 					echo '<td align="right">&nbsp;'.(isset($fileinfo['bitrate']) ? BitrateText($fileinfo['bitrate'] / 1000, 0, ((isset($fileinfo['audio']['bitrate_mode']) && ($fileinfo['audio']['bitrate_mode'] == 'vbr')) ? true : false)) : '-').'</td>';
 					echo '<td align="left">&nbsp;'.(isset($fileinfo['comments_html']['artist']) ? implode('<br>', $fileinfo['comments_html']['artist']) : ((isset($fileinfo['video']['resolution_x']) && isset($fileinfo['video']['resolution_y'])) ? $fileinfo['video']['resolution_x'].'x'.$fileinfo['video']['resolution_y'] : '')).'</td>';
 					echo '<td align="left">&nbsp;'.(isset($fileinfo['comments_html']['title'])  ? implode('<br>', $fileinfo['comments_html']['title'])  :  (isset($fileinfo['video']['frame_rate'])                                                 ? number_format($fileinfo['video']['frame_rate'], 3).'fps'                  : '')).'</td>';
-					if (isset($_REQUEST['ShowMD5'])) {
+					if (isset($_REQUEST['ShowMD5']) && GETID3_DEMO_BROWSE_ALLOW_MD5_LINK) {
 						echo '<td align="left"><tt>'.(isset($fileinfo['md5_file'])        ? $fileinfo['md5_file']        : '&nbsp;').'</tt></td>';
 						echo '<td align="left"><tt>'.(isset($fileinfo['md5_data'])        ? $fileinfo['md5_data']        : '&nbsp;').'</tt></td>';
 						echo '<td align="left"><tt>'.(isset($fileinfo['md5_data_source']) ? $fileinfo['md5_data_source'] : '&nbsp;').'</tt></td>';
@@ -287,35 +297,39 @@ if (isset($_REQUEST['filename'])) {
 					echo '<td align="left">&nbsp;';
 					if (!empty($fileinfo['warning'])) {
 						$FilesWithWarnings++;
-						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['warning']))), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['warning']), ENT_QUOTES).'">warning</a><br>';
+						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['warning']))), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['warning']), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'">warning</a><br>';
 					}
 					if (!empty($fileinfo['error'])) {
 						$FilesWithErrors++;
-						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['error']))),   ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['error']),   ENT_QUOTES).'">error</a><br>';
+						echo '<a href="#" onClick="alert(\''.htmlentities(str_replace("'", "\\'", preg_replace('#[\r\n\t]+#', ' ', implode('\\n', $fileinfo['error']))),   ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'\'); return false;" title="'.htmlentities(implode("; \n", $fileinfo['error']),   ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'">error</a><br>';
 					}
 					echo '</td>';
 
-					echo '<td align="left">&nbsp;';
-					$fileinfo['fileformat'] = (isset($fileinfo['fileformat']) ? $fileinfo['fileformat'] : '');
-					switch ($fileinfo['fileformat']) {
-						case 'mp3':
-						case 'mp2':
-						case 'mp1':
-						case 'flac':
-						case 'mpc':
-						case 'real':
-							echo '<a href="'.htmlentities($writescriptfilename.'?Filename='.urlencode($dirname.$filename), ENT_QUOTES).'" title="Edit tags">edit&nbsp;tags</a>';
-							break;
-						case 'ogg':
-							if (isset($fileinfo['audio']['dataformat']) && ($fileinfo['audio']['dataformat'] == 'vorbis')) {
-								echo '<a href="'.htmlentities($writescriptfilename.'?Filename='.urlencode($dirname.$filename), ENT_QUOTES).'" title="Edit tags">edit&nbsp;tags</a>';
-							}
-							break;
-						default:
-							break;
+					if (GETID3_DEMO_BROWSE_ALLOW_EDIT_LINK) {
+						echo '<td align="left">&nbsp;';
+						$fileinfo['fileformat'] = (isset($fileinfo['fileformat']) ? $fileinfo['fileformat'] : '');
+						switch ($fileinfo['fileformat']) {
+							case 'mp3':
+							case 'mp2':
+							case 'mp1':
+							case 'flac':
+							case 'mpc':
+							case 'real':
+								echo '<a href="'.htmlentities($writescriptfilename.'?Filename='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" title="Edit tags">edit&nbsp;tags</a>';
+								break;
+							case 'ogg':
+								if (isset($fileinfo['audio']['dataformat']) && ($fileinfo['audio']['dataformat'] == 'vorbis')) {
+									echo '<a href="'.htmlentities($writescriptfilename.'?Filename='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" title="Edit tags">edit&nbsp;tags</a>';
+								}
+								break;
+							default:
+								break;
+						}
+						echo '</td>';
+					}
+					if (GETID3_DEMO_BROWSE_ALLOW_DELETE_LINK) {
+						echo '<td align="left">&nbsp;<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory).'&deletefile='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" onClick="return confirm(\'Are you sure you want to delete '.addslashes(htmlentities($dirname.$filename)).'? \n(this action cannot be un-done)\');" title="'.htmlentities('Permanently delete '."\n".$filename."\n".' from'."\n".' '.$dirname, ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'">delete</a></td>';
 					}
-					echo '</td>';
-					echo '<td align="left">&nbsp;<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory).'&deletefile='.urlencode($dirname.$filename), ENT_QUOTES).'" onClick="return confirm(\'Are you sure you want to delete '.addslashes(htmlentities($dirname.$filename)).'? \n(this action cannot be un-done)\');" title="'.htmlentities('Permanently delete '."\n".$filename."\n".' from'."\n".' '.$dirname, ENT_QUOTES).'">delete</a></td>';
 					echo '</tr>';
 				}
 			}
@@ -324,7 +338,9 @@ if (isset($_REQUEST['filename'])) {
 				uksort($DirectoryContents[$dirname]['other'], 'MoreNaturalSort');
 				foreach ($DirectoryContents[$dirname]['other'] as $filename => $fileinfo) {
 					echo '<tr bgcolor="#'.(($rowcounter++ % 2) ? $getID3checkColor_UnknownDark : $getID3checkColor_UnknownLight).'">';
-					echo '<td><a href="'.htmlentities($_SERVER['PHP_SELF'].'?filename='.urlencode($dirname.$filename), ENT_QUOTES).'"><i>'.htmlentities($filename).'</i></a></td>';
+					$escaped_filename = htmlentities($filename, ENT_SUBSTITUTE, 'ISO-8859-1');
+					$escaped_filename = ($escaped_filename ? $escaped_filename : rawurlencode($filename));
+					echo '<td><a href="'.htmlentities($_SERVER['PHP_SELF'].'?filename='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'"><i>'.$escaped_filename.'</i></a></td>';
 					echo '<td align="right">&nbsp;'.(isset($fileinfo['filesize']) ? number_format($fileinfo['filesize']) : '-').'</td>';
 					echo '<td align="right">&nbsp;'.NiceDisplayFiletypeFormat($fileinfo).'</td>';
 					echo '<td align="right">&nbsp;'.(isset($fileinfo['playtime_string']) ? $fileinfo['playtime_string'] : '-').'</td>';
@@ -338,18 +354,22 @@ if (isset($_REQUEST['filename'])) {
 					echo '<td align="left">&nbsp;';
 					if (!empty($fileinfo['warning'])) {
 						$FilesWithWarnings++;
-						echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['warning']), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['warning']), ENT_QUOTES).'">warning</a><br>';
+						echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['warning']), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['warning']), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'">warning</a><br>';
 					}
 					if (!empty($fileinfo['error'])) {
 						if ($fileinfo['error'][0] != 'unable to determine file format') {
 							$FilesWithErrors++;
-							echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['error']), ENT_QUOTES).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['error']), ENT_QUOTES).'">error</a><br>';
+							echo '<a href="#" onClick="alert(\''.htmlentities(implode('\\n', $fileinfo['error']), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'\'); return false;" title="'.htmlentities(implode("\n", $fileinfo['error']), ENT_QUOTES | ENT_SUBSTITUTE, $PageEncoding).'">error</a><br>';
 						}
 					}
 					echo '</td>';
 
-					echo '<td align="left">&nbsp;</td>'; // Edit
-					echo '<td align="left">&nbsp;<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory).'&deletefile='.urlencode($dirname.$filename), ENT_QUOTES).'" onClick="return confirm(\'Are you sure you want to delete '.addslashes($dirname.$filename).'? \n(this action cannot be un-done)\');" title="Permanently delete '.addslashes($dirname.$filename).'">delete</a></td>';
+					if (GETID3_DEMO_BROWSE_ALLOW_EDIT_LINK) {
+						echo '<td align="left">&nbsp;</td>'; // Edit
+					}
+					if (GETID3_DEMO_BROWSE_ALLOW_DELETE_LINK) {
+						echo '<td align="left">&nbsp;<a href="'.htmlentities($_SERVER['PHP_SELF'].'?listdirectory='.urlencode($listdirectory).'&deletefile='.urlencode($dirname.$filename), ENT_QUOTES | ENT_SUBSTITUTE, 'ISO-8859-1').'" onClick="return confirm(\'Are you sure you want to delete '.addslashes($dirname.$filename).'? \n(this action cannot be un-done)\');" title="Permanently delete '.addslashes($dirname.$filename).'">delete</a></td>';
+					}
 					echo '</tr>';
 				}
 			}
@@ -488,12 +508,7 @@ function table_var_dump($variable, $wrap_in_td=false, $encoding='ISO-8859-1') {
 			break;
 
 		case 'string':
-			//$variable = str_replace("\x00", ' ', $variable);
-			//$varlen = strlen($variable);
-			//for ($i = 0; $i < $varlen; $i++) {
-			//	$returnstring .= htmlentities($variable{$i}, ENT_QUOTES, $encoding);
-			//}
-			$returnstring = htmlentities($variable, ENT_QUOTES, $encoding);
+			$returnstring = htmlentities($variable, ENT_QUOTES | ENT_SUBSTITUTE, $encoding);
 			$returnstring = ($wrap_in_td ? '<td class="dump_string">' : '').nl2br($returnstring).($wrap_in_td ? '</td>'."\n" : '');
 			break;
 
@@ -593,5 +608,3 @@ function PoweredBygetID3($string='') {
 	}
 	return str_replace('<!--GETID3VER-->', $getID3->version(), $string);
 }
-
-?>
\ No newline at end of file
