@@ -70,6 +70,4 @@ function GetMIMEtype($filename) {
 	return $DeterminedMIMEtype;
 }
 
-?>
-</body>
-</html>
\ No newline at end of file
+echo '</body></html>';
\ No newline at end of file
