@@ -13,27 +13,31 @@
 //                                                            ///
 /////////////////////////////////////////////////////////////////
 
+define('GETID3_MIDI_MAGIC_MTHD', 'MThd'); // MIDI file header magic
+define('GETID3_MIDI_MAGIC_MTRK', 'MTrk'); // MIDI track header magic
 
-class getid3_midi
+class getid3_midi extends getid3_handler
 {
+	public $scanwholefile = true;
 
-	function getid3_midi(&$fd, &$ThisFileInfo, $scanwholefile=true) {
+	public function Analyze() {
+		$info = &$this->getid3->info;
 
 		// shortcut
-		$ThisFileInfo['midi']['raw'] = array();
-		$thisfile_midi               = &$ThisFileInfo['midi'];
+		$info['midi']['raw'] = array();
+		$thisfile_midi               = &$info['midi'];
 		$thisfile_midi_raw           = &$thisfile_midi['raw'];
 
-		$ThisFileInfo['fileformat']          = 'midi';
-		$ThisFileInfo['audio']['dataformat'] = 'midi';
+		$info['fileformat']          = 'midi';
+		$info['audio']['dataformat'] = 'midi';
 
-		fseek($fd, $ThisFileInfo['avdataoffset'], SEEK_SET);
-		$MIDIdata = fread($fd, GETID3_FREAD_BUFFER_SIZE);
+		fseek($this->getid3->fp, $info['avdataoffset'], SEEK_SET);
+		$MIDIdata = fread($this->getid3->fp, $this->getid3->fread_buffer_size());
 		$offset = 0;
 		$MIDIheaderID = substr($MIDIdata, $offset, 4); // 'MThd'
-		if ($MIDIheaderID != 'MThd') {
-			$ThisFileInfo['error'][] = 'Expecting "MThd" at offset '.$ThisFileInfo['avdataoffset'].', found "'.$MIDIheaderID.'"';
-			unset($ThisFileInfo['fileformat']);
+		if ($MIDIheaderID != GETID3_MIDI_MAGIC_MTHD) {
+			$info['error'][] = 'Expecting "'.getid3_lib::PrintHexBytes(GETID3_MIDI_MAGIC_MTHD).'" at offset '.$info['avdataoffset'].', found "'.getid3_lib::PrintHexBytes($MIDIheaderID).'"';
+			unset($info['fileformat']);
 			return false;
 		}
 		$offset += 4;
@@ -47,35 +51,42 @@ class getid3_midi
 		$offset += 2;
 
 		for ($i = 0; $i < $thisfile_midi_raw['tracks']; $i++) {
-			if ((strlen($MIDIdata) - $offset) < 8) {
-				$MIDIdata .= fread($fd, GETID3_FREAD_BUFFER_SIZE);
+			while ((strlen($MIDIdata) - $offset) < 8) {
+				if ($buffer = fread($this->getid3->fp, $this->getid3->fread_buffer_size())) {
+					$MIDIdata .= $buffer;
+				} else {
+					$info['warning'][] = 'only processed '.($i - 1).' of '.$thisfile_midi_raw['tracks'].' tracks';
+					$info['error'][] = 'Unabled to read more file data at '.ftell($this->getid3->fp).' (trying to seek to : '.$offset.'), was expecting at least 8 more bytes';
+					return false;
+				}
 			}
 			$trackID = substr($MIDIdata, $offset, 4);
 			$offset += 4;
-			if ($trackID == 'MTrk') {
+			if ($trackID == GETID3_MIDI_MAGIC_MTRK) {
 				$tracksize = getid3_lib::BigEndian2Int(substr($MIDIdata, $offset, 4));
 				$offset += 4;
-				// $thisfile_midi['tracks'][$i]['size'] = $tracksize;
+				//$thisfile_midi['tracks'][$i]['size'] = $tracksize;
 				$trackdataarray[$i] = substr($MIDIdata, $offset, $tracksize);
 				$offset += $tracksize;
 			} else {
-				$ThisFileInfo['error'][] = 'Expecting "MTrk" at '.$offset.', found '.$trackID.' instead';
+				$info['error'][] = 'Expecting "'.getid3_lib::PrintHexBytes(GETID3_MIDI_MAGIC_MTRK).'" at '.($offset - 4).', found "'.getid3_lib::PrintHexBytes($trackID).'" instead';
 				return false;
 			}
 		}
 
 		if (!isset($trackdataarray) || !is_array($trackdataarray)) {
-			$ThisFileInfo['error'][] = 'Cannot find MIDI track information';
+			$info['error'][] = 'Cannot find MIDI track information';
 			unset($thisfile_midi);
-			unset($ThisFileInfo['fileformat']);
+			unset($info['fileformat']);
 			return false;
 		}
 
-		if ($scanwholefile) { // this can take quite a long time, so have the option to bypass it if speed is very important
+		if ($this->scanwholefile) { // this can take quite a long time, so have the option to bypass it if speed is very important
 			$thisfile_midi['totalticks']      = 0;
-			$ThisFileInfo['playtime_seconds'] = 0;
+			$info['playtime_seconds'] = 0;
 			$CurrentMicroSecondsPerBeat       = 500000; // 120 beats per minute;  60,000,000 microseconds per minute -> 500,000 microseconds per beat
 			$CurrentBeatsPerMinute            = 120;    // 120 beats per minute;  60,000,000 microseconds per minute -> 500,000 microseconds per beat
+			$MicroSecondsPerQuarterNoteAfter  = array ();
 
 			foreach ($trackdataarray as $tracknumber => $trackdata) {
 
@@ -214,7 +225,7 @@ class getid3_midi
 							case 0x51: // Tempo: microseconds / quarter note
 								$CurrentMicroSecondsPerBeat = getid3_lib::BigEndian2Int(substr($METAeventData, 0, $METAeventLength));
 								if ($CurrentMicroSecondsPerBeat == 0) {
-									$ThisFileInfo['error'][] = 'Corrupt MIDI file: CurrentMicroSecondsPerBeat == zero';
+									$info['error'][] = 'Corrupt MIDI file: CurrentMicroSecondsPerBeat == zero';
 									return false;
 								}
 								$thisfile_midi_raw['events'][$tracknumber][$CumulativeDeltaTime]['us_qnote'] = $CurrentMicroSecondsPerBeat;
@@ -257,13 +268,13 @@ class getid3_midi
 								break;
 
 							default:
-								$ThisFileInfo['warning'][] = 'Unhandled META Event Command: '.$METAeventCommand;
+								$info['warning'][] = 'Unhandled META Event Command: '.$METAeventCommand;
 								break;
 						}
 
 					} else {
 
-						$ThisFileInfo['warning'][] = 'Unhandled MIDI Event ID: '.$MIDIevents[$tracknumber][$eventid]['eventid'].' + Channel ID: '.$MIDIevents[$tracknumber][$eventid]['channel'];
+						$info['warning'][] = 'Unhandled MIDI Event ID: '.$MIDIevents[$tracknumber][$eventid]['eventid'].' + Channel ID: '.$MIDIevents[$tracknumber][$eventid]['channel'];
 
 					}
 				}
@@ -283,11 +294,11 @@ class getid3_midi
 				if ($thisfile_midi['totalticks'] > $tickoffset) {
 
 					if ($thisfile_midi_raw['ticksperqnote'] == 0) {
-						$ThisFileInfo['error'][] = 'Corrupt MIDI file: ticksperqnote == zero';
+						$info['error'][] = 'Corrupt MIDI file: ticksperqnote == zero';
 						return false;
 					}
 
-					$ThisFileInfo['playtime_seconds'] += (($tickoffset - $previoustickoffset) / $thisfile_midi_raw['ticksperqnote']) * ($prevmicrosecondsperbeat / 1000000);
+					$info['playtime_seconds'] += (($tickoffset - $previoustickoffset) / $thisfile_midi_raw['ticksperqnote']) * ($prevmicrosecondsperbeat / 1000000);
 
 					$prevmicrosecondsperbeat = $microsecondsperbeat;
 					$previoustickoffset = $tickoffset;
@@ -296,17 +307,18 @@ class getid3_midi
 			if ($thisfile_midi['totalticks'] > $previoustickoffset) {
 
 				if ($thisfile_midi_raw['ticksperqnote'] == 0) {
-					$ThisFileInfo['error'][] = 'Corrupt MIDI file: ticksperqnote == zero';
+					$info['error'][] = 'Corrupt MIDI file: ticksperqnote == zero';
 					return false;
 				}
 
-				$ThisFileInfo['playtime_seconds'] += (($thisfile_midi['totalticks'] - $previoustickoffset) / $thisfile_midi_raw['ticksperqnote']) * ($microsecondsperbeat / 1000000);
+				$info['playtime_seconds'] += (($thisfile_midi['totalticks'] - $previoustickoffset) / $thisfile_midi_raw['ticksperqnote']) * ($microsecondsperbeat / 1000000);
 
 			}
 		}
 
-		if ($ThisFileInfo['playtime_seconds'] > 0) {
-			$ThisFileInfo['bitrate'] = (($ThisFileInfo['avdataend'] - $ThisFileInfo['avdataoffset']) * 8) / $ThisFileInfo['playtime_seconds'];
+
+		if (!empty($info['playtime_seconds'])) {
+			$info['bitrate'] = (($info['avdataend'] - $info['avdataoffset']) * 8) / $info['playtime_seconds'];
 		}
 
 		if (!empty($thisfile_midi['lyrics'])) {
@@ -316,7 +328,7 @@ class getid3_midi
 		return true;
 	}
 
-	function GeneralMIDIinstrumentLookup($instrumentid) {
+	public function GeneralMIDIinstrumentLookup($instrumentid) {
 
 		$begin = __LINE__;
 
@@ -456,7 +468,7 @@ class getid3_midi
 		return getid3_lib::EmbeddedLookup($instrumentid, $begin, __LINE__, __FILE__, 'GeneralMIDIinstrument');
 	}
 
-	function GeneralMIDIpercussionLookup($instrumentid) {
+	public function GeneralMIDIpercussionLookup($instrumentid) {
 
 		$begin = __LINE__;
 
@@ -515,6 +527,3 @@ class getid3_midi
 	}
 
 }
-
-
-?>
\ No newline at end of file
