@@ -14,37 +14,38 @@
 /////////////////////////////////////////////////////////////////
 
 
-class getid3_png
+class getid3_png extends getid3_handler
 {
 
-	function getid3_png(&$fd, &$ThisFileInfo) {
+	public function Analyze() {
+		$info = &$this->getid3->info;
 
-	    // shortcut
-	    $ThisFileInfo['png'] = array();
-	    $thisfile_png = &$ThisFileInfo['png'];
+		// shortcut
+		$info['png'] = array();
+		$thisfile_png = &$info['png'];
 
-		$ThisFileInfo['fileformat']          = 'png';
-		$ThisFileInfo['video']['dataformat'] = 'png';
-		$ThisFileInfo['video']['lossless']   = false;
+		$info['fileformat']          = 'png';
+		$info['video']['dataformat'] = 'png';
+		$info['video']['lossless']   = false;
 
-		fseek($fd, $ThisFileInfo['avdataoffset'], SEEK_SET);
-		$PNGfiledata = fread($fd, GETID3_FREAD_BUFFER_SIZE);
+		fseek($this->getid3->fp, $info['avdataoffset'], SEEK_SET);
+		$PNGfiledata = fread($this->getid3->fp, $this->getid3->fread_buffer_size());
 		$offset = 0;
 
 		$PNGidentifier = substr($PNGfiledata, $offset, 8); // $89 $50 $4E $47 $0D $0A $1A $0A
 		$offset += 8;
 
 		if ($PNGidentifier != "\x89\x50\x4E\x47\x0D\x0A\x1A\x0A") {
-			$ThisFileInfo['error'][] = 'First 8 bytes of file ('.getid3_lib::PrintHexBytes($PNGidentifier).') did not match expected PNG identifier';
-			unset($ThisFileInfo['fileformat']);
+			$info['error'][] = 'First 8 bytes of file ('.getid3_lib::PrintHexBytes($PNGidentifier).') did not match expected PNG identifier';
+			unset($info['fileformat']);
 			return false;
 		}
 
-		while (((ftell($fd) - (strlen($PNGfiledata) - $offset)) < $ThisFileInfo['filesize'])) {
+		while (((ftell($this->getid3->fp) - (strlen($PNGfiledata) - $offset)) < $info['filesize'])) {
 			$chunk['data_length'] = getid3_lib::BigEndian2Int(substr($PNGfiledata, $offset, 4));
 			$offset += 4;
-			while (((strlen($PNGfiledata) - $offset) < ($chunk['data_length'] + 4)) && (ftell($fd) < $ThisFileInfo['filesize'])) {
-				$PNGfiledata .= fread($fd, GETID3_FREAD_BUFFER_SIZE);
+			while (((strlen($PNGfiledata) - $offset) < ($chunk['data_length'] + 4)) && (ftell($this->getid3->fp) < $info['filesize'])) {
+				$PNGfiledata .= fread($this->getid3->fp, $this->getid3->fread_buffer_size());
 			}
 			$chunk['type_text']   =               substr($PNGfiledata, $offset, 4);
 			$offset += 4;
@@ -80,10 +81,10 @@ class getid3_png
 					$thisfile_png_chunk_type_text['color_type']['true_color']  = (bool) ($thisfile_png_chunk_type_text['raw']['color_type'] & 0x02);
 					$thisfile_png_chunk_type_text['color_type']['alpha']       = (bool) ($thisfile_png_chunk_type_text['raw']['color_type'] & 0x04);
 
-					$ThisFileInfo['video']['resolution_x']    = $thisfile_png_chunk_type_text['width'];
-					$ThisFileInfo['video']['resolution_y']    = $thisfile_png_chunk_type_text['height'];
+					$info['video']['resolution_x']    = $thisfile_png_chunk_type_text['width'];
+					$info['video']['resolution_y']    = $thisfile_png_chunk_type_text['height'];
 
-					$ThisFileInfo['video']['bits_per_sample'] = $this->IHDRcalculateBitsPerSample($thisfile_png_chunk_type_text['raw']['color_type'], $thisfile_png_chunk_type_text['raw']['bit_depth']);
+					$info['video']['bits_per_sample'] = $this->IHDRcalculateBitsPerSample($thisfile_png_chunk_type_text['raw']['color_type'], $thisfile_png_chunk_type_text['raw']['bit_depth']);
 					break;
 
 
@@ -123,10 +124,10 @@ class getid3_png
 
 						case 4:
 						case 6:
-							$ThisFileInfo['error'][] = 'Invalid color_type in tRNS chunk: '.$thisfile_png['IHDR']['raw']['color_type'];
+							$info['error'][] = 'Invalid color_type in tRNS chunk: '.$thisfile_png['IHDR']['raw']['color_type'];
 
 						default:
-							$ThisFileInfo['warning'][] = 'Unhandled color_type in tRNS chunk: '.$thisfile_png['IHDR']['raw']['color_type'];
+							$info['warning'][] = 'Unhandled color_type in tRNS chunk: '.$thisfile_png['IHDR']['raw']['color_type'];
 							break;
 					}
 					break;
@@ -429,7 +430,7 @@ class getid3_png
 				default:
 					//unset($chunk['data']);
 					$thisfile_png_chunk_type_text['header'] = $chunk;
-					$ThisFileInfo['warning'][] = 'Unhandled chunk type: '.$chunk['type_text'];
+					$info['warning'][] = 'Unhandled chunk type: '.$chunk['type_text'];
 					break;
 			}
 		}
@@ -437,7 +438,7 @@ class getid3_png
 		return true;
 	}
 
-	function PNGsRGBintentLookup($sRGB) {
+	public function PNGsRGBintentLookup($sRGB) {
 		static $PNGsRGBintentLookup = array(
 			0 => 'Perceptual',
 			1 => 'Relative colorimetric',
@@ -447,14 +448,14 @@ class getid3_png
 		return (isset($PNGsRGBintentLookup[$sRGB]) ? $PNGsRGBintentLookup[$sRGB] : 'invalid');
 	}
 
-	function PNGcompressionMethodLookup($compressionmethod) {
+	public function PNGcompressionMethodLookup($compressionmethod) {
 		static $PNGcompressionMethodLookup = array(
 			0 => 'deflate/inflate'
 		);
 		return (isset($PNGcompressionMethodLookup[$compressionmethod]) ? $PNGcompressionMethodLookup[$compressionmethod] : 'invalid');
 	}
 
-	function PNGpHYsUnitLookup($unitid) {
+	public function PNGpHYsUnitLookup($unitid) {
 		static $PNGpHYsUnitLookup = array(
 			0 => 'unknown',
 			1 => 'meter'
@@ -462,7 +463,7 @@ class getid3_png
 		return (isset($PNGpHYsUnitLookup[$unitid]) ? $PNGpHYsUnitLookup[$unitid] : 'invalid');
 	}
 
-	function PNGoFFsUnitLookup($unitid) {
+	public function PNGoFFsUnitLookup($unitid) {
 		static $PNGoFFsUnitLookup = array(
 			0 => 'pixel',
 			1 => 'micrometer'
@@ -470,7 +471,7 @@ class getid3_png
 		return (isset($PNGoFFsUnitLookup[$unitid]) ? $PNGoFFsUnitLookup[$unitid] : 'invalid');
 	}
 
-	function PNGpCALequationTypeLookup($equationtype) {
+	public function PNGpCALequationTypeLookup($equationtype) {
 		static $PNGpCALequationTypeLookup = array(
 			0 => 'Linear mapping',
 			1 => 'Base-e exponential mapping',
@@ -480,7 +481,7 @@ class getid3_png
 		return (isset($PNGpCALequationTypeLookup[$equationtype]) ? $PNGpCALequationTypeLookup[$equationtype] : 'invalid');
 	}
 
-	function PNGsCALUnitLookup($unitid) {
+	public function PNGsCALUnitLookup($unitid) {
 		static $PNGsCALUnitLookup = array(
 			0 => 'meter',
 			1 => 'radian'
@@ -488,7 +489,7 @@ class getid3_png
 		return (isset($PNGsCALUnitLookup[$unitid]) ? $PNGsCALUnitLookup[$unitid] : 'invalid');
 	}
 
-	function IHDRcalculateBitsPerSample($color_type, $bit_depth) {
+	public function IHDRcalculateBitsPerSample($color_type, $bit_depth) {
 		switch ($color_type) {
 			case 0: // Each pixel is a grayscale sample.
 				return $bit_depth;
@@ -514,6 +515,3 @@ class getid3_png
 	}
 
 }
-
-
-?>
\ No newline at end of file
